; data: ARRAY[nx,ny] OF FLOAT
; bpm: POINTER TO ARRAY[nx,ny] OF FLOAT
PRO LDRS_BPM_PAIR_INTERPOLATION, data, bpm
; check the dimensions of the data and the bpm
ds = SIZE(data, /DIMENSIONS)
bs = SIZE(*bpm, /DIMENSIONS)
IF (ds[0] NE bs[0]) OR (ds[1] NE bs[1]) THEN BEGIN
    ; the dimensions differ => not compatible
    PRINT, 'LDRS_BPM_PAIR_INTERPOLATION: the dimensions of the data and the BPM differ!'
    RETURN
ENDIF
nx = ds[0]
ny = ds[1]
; bi are the 1-dimensional indices of the bad pixels
bi = WHERE(*bpm EQ 0.0, count)
; replacement pixels are positioned at the 8 directions with
; the bad pixel at the center:
;  0 | 1 | 2
;  3 | X | 4
;  5 | 6 | 7
; rpf: ARRAY[8] OF INT   ; replacement pixel flag
; rpw: ARRAY[8] OF FLOAT ; replacement pixel weight/dist
; rpv: ARRAY[8] OF FLOAT ; replacement pixel value
rpf = INTARR(8)
rpw = FLTARR(8)
rpv = FLTARR(8)
rpdx = [-1,0,1,-1,1,-1,0,1]
rpdy = [1,1,1,0,0,-1,-1,-1]
FOR i=0,count-1 DO BEGIN
    ; the 2-dimensional indices of a bad pixel are (bx,by)
    bx = bi[i] MOD nx
    by = bi[i]/nx
    rpf[*] = 0
    rpw[*] = 0.0
    rpv[*] = 0.0
    ; search the replacement pixels
    FOR j=0,7 DO BEGIN
        rx = bx + rpdx[j]
        ry = by + rpdy[j]
        rpw[j] = 1.0
        WHILE rpf[j] EQ 0 DO BEGIN
            IF (rx LT 0) OR (rx GE nx) THEN BREAK
            IF (ry LT 0) OR (ry GE ny) THEN BREAK
            IF (*bpm)[rx,ry] EQ 0.0 THEN BEGIN
                ; the replacement pixel is also a bad pixel
                ; => move to the next replacement pixel
                rx += rpdx[j]
                ry += rpdy[j]
                rpw[j] += 1.0
                CONTINUE
            ENDIF
            ; we found an useful replacement pixel
            rpf[j] = 1
            rpv[j] = data[rx,ry]
        ENDWHILE
    ENDFOR
    IF (bx EQ 1038) AND (by EQ 1007) THEN BEGIN
        PRINT, 'original values for the bad pixel at ', bx, ', ', by
        FOR j=0,7 DO PRINT, '   j=', j, ', rpv=', rpv[j], ', rpw=', rpw[j]
    ENDIF
    ; recalculate the replacement values and weights for pairs
    FOR j=0,3 DO BEGIN
        k = 7 - j
        IF rpf[j] EQ 1 THEN BEGIN
            IF rpf[k] EQ 1 THEN BEGIN
                ; we have a pair, recalculate the value and weight
                rpv[j] = (rpv[j]*rpw[k] + rpv[k]*rpw[j])/(rpw[j] + rpw[k])
                rpw[j] = rpw[j]^2 + rpw[k]^2
                ; remove the weight at k
                rpf[k] = 0
                rpw[k] = !VALUES.F_NAN
                rpv[k] = 0.0
            ENDIF ELSE BEGIN
                ; only a value at j does exist, recalculate the weight
                rpw[j] = 2*rpw[j]^2 + 1 ; a little bit worse than a pair!
                ; remove the weight at k
                rpw[k] = !VALUES.F_NAN
            ENDELSE
        ENDIF ELSE BEGIN
            IF rpf[k] EQ 1 THEN BEGIN
                ; only a value at k does exist, recalculate the weight
                rpw[k] = 2*rpw[k]^2 + 1 ; a little bit worse than a pair!
                ; remove the weight at j
                rpw[j] = !VALUES.F_NAN
            ENDIF ELSE BEGIN
                ; no value at j or k, remove both weights
                rpw[j] = !VALUES.F_NAN
                rpw[k] = !VALUES.F_NAN
            ENDELSE
        ENDELSE
    ENDFOR
    ; find the values with the smallest weights an use up to 2 values
    vl = SORT(rpw)
    ; we have to distinguish between three different cases:
    ; 1. no replacement value dies exist (extremely rare)
    ; 2. only one replacement value does exist (very rare)
    ; 3. two or more replacement values do exist (normal case)
    IF (bx EQ 1038) AND (by EQ 1007) THEN BEGIN
        PRINT, ' pairs:'
        FOR j=0,7 DO PRINT, '   j=', j, ', rpv=', rpv[j], ', rpw=', rpw[j]
        PRINT, ' sort=', vl
        PRINT, ' old value = ', data[bx,by]
    ENDIF
    IF rpf[vl[0]] NE !VALUES.F_NAN THEN BEGIN
        v1 = rpv[vl[0]]
        w1 = rpw[vl[0]]
        IF rpf[vl[1]] NE !VALUES.F_NAN THEN BEGIN
            v2 = rpv[vl[1]]
            w2 = rpw[vl[1]]
            data[bx,by] = (v1*w2 + v2*w1)/(w1 + w2)
        ENDIF ELSE BEGIN
            data[bx,by] = v1
        ENDELSE
    ENDIF
    IF (bx EQ 1038) AND (by EQ 1007) THEN PRINT, ' new value = ', data[bx,by]
ENDFOR
END
