FUNCTION ldrs_transform_fft, data, angle, transX, transY
PRINT, 'ldrs_transform_fft, angle = ', angle, ', translate = ', transX, transY
phi = -angle
s = SIZE(data, /DIMENSIONS)
nx = 2*s[0] ; size of the temporary array!
ny = 2*s[1]
img = COMPLEXARR(nx, ny)
PRINT, '  phi old=', phi
IF phi LT -45.0 THEN phi += 360.0
IF phi GT 315.0 THEN phi -= 360.0
PRINT, '  phi before=', phi
IF phi LE 45.0 THEN BEGIN
    img[nx/4:nx/4 + s[0] -1, ny/4:ny/4 + s[1] - 1] = data[*,*]
ENDIF ELSE BEGIN
    ; ROTATE rotates about (nx-1)/2, ROT about (nx/2) !!!
    hdata = FLTARR(s[0] + 1, s[1] + 1)
    hdata[0:s[0]-1, 0:s[1]-1] = data[*,*]
    IF phi GT 225.0 THEN BEGIN
        hdata = ROTATE(hdata, 3)
        phi -= 270.0
    ENDIF ELSE BEGIN
        IF phi GT 135.0 THEN BEGIN
            hdata = ROTATE(hdata, 2)
            phi -= 180.0
        ENDIF ELSE BEGIN
            IF phi GT 45.0 THEN BEGIN
                hdata = ROTATE(hdata, 1)
                phi -= 90.0
            ENDIF
        ENDELSE
    ENDELSE
    img[nx/4:nx/4 + s[0] -1, ny/4:ny/4 + s[1] - 1] = hdata[0:s[0]-1, 0:s[1]-1]
ENDELSE
PRINT, '  phi new=', phi
ua = FINDGEN(nx)                ; 0 1 2 3  4  5  6  7
ua[nx/2:nx-1] -= nx             ; 0 1 2 3 -4 -3 -2 -1
ua[*] /= FLOAT(nx)
va = FINDGEN(ny)                ; 0 1 2 3  4  5  6  7
va[ny/2:ny-1] -= ny             ; 0 1 2 3 -4 -3 -2 -1
va[*] /= FLOAT(ny)
PRINT, '  first shear about', phi/2
; first shear about 
; |x1| = | 1  -tan(phi/2) | |x|
; |y1| = | 0       1      | |y|
img = FFT(img, -1, DIMENSION=1, /OVERWRITE)
skew = -tan(!PI*phi/360.0) ; 360.0 is correct (tan(phi/2)!)
shift = -skew*FLOAT(ny)/2.0
PRINT, '  shearing..., skew=', skew, ', shift=', shift
FOR y=0,ny-1 DO BEGIN
    f = -2.0*!PI*shift
    ca = COMPLEX(cos(ua*f), sin(ua*f))
    img[*,y] *= ca[*]
    shift += skew
ENDFOR
img = FFT(img, 1, DIMENSION=1, /INVERSE, /OVERWRITE)
;img[*,*] *= COMPLEX(1.0/nx,0.0)
PRINT, '  second shear about', phi
; second shear about
; |x2| = | 1         0 ||x1|
; |y2| = | sin(phi)  1 ||y1|
img = FFT(img, -1, DIMENSION=2, /OVERWRITE)
skew = sin(!PI*phi/180.0)
shift = -skew*FLOAT(nx)/2.0
PRINT, '  shearing..., skew=', skew, ', shift=', shift
FOR x=0,nx-1 DO BEGIN
    f = -2.0*!PI*shift
    ca = COMPLEX(cos(va*f), sin(va*f))
    img[x,*] *= ca[*]
    shift += skew
ENDFOR
img = FFT(img, 1, DIMENSION=2, /INVERSE, /OVERWRITE)
;img[*,*] *= COMPLEX(1.0/ny,0.0)
PRINT, '  third shear about', phi/2
; third shear about 
; |x3| = | 1  -tan(phi/2) | |x2|
; |y3| = | 0       1      | |y2|
img = FFT(img, -1, DIMENSION=1, /OVERWRITE)
skew = -tan(!PI*phi/360.0) ; 360.0 is correct (tan(angle/2)!)
shift = -skew*FLOAT(ny)/2.0
PRINT, '  shearing..., skew=', skew, ', shift=', shift
FOR y=0,ny-1 DO BEGIN
    f = -2.0*!PI*shift
    ca = COMPLEX(cos(ua*f), sin(ua*f))
    img[*,y] *= ca[*]
    shift += skew
ENDFOR
img = FFT(img, 1, DIMENSION=1, /INVERSE, /OVERWRITE)
; translate about (transX, transY)
IF (transX NE 0.0) OR (transY NE 0.0) THEN BEGIN
    img = FFT(img, /OVERWRITE)
    ; translate in x direction
    IF transX NE 0.0 THEN BEGIN
        FOR x=0, nx-1 DO BEGIN
            IF x LT (nx/2) THEN u = FLOAT(x)/FLOAT(nx) ELSE u = FLOAT(x-nx)/FLOAT(nx)
            p = -2.0*!PI*transX*u
            c = COMPLEX(COS(p), SIN(p))
            FOR y=0,ny-1 DO img[x,y] = img[x,y]*c
        ENDFOR
    ENDIF
    ; translate in y direction
    IF transY NE 0.0 THEN BEGIN
        FOR y=0, ny-1 DO BEGIN
            IF y LT (ny/2) THEN v = FLOAT(y)/FLOAT(ny) ELSE v = FLOAT(y-ny)/FLOAT(ny)
            p = -2.0*!PI*transY*v
            c = COMPLEX(COS(p), SIN(p))
            FOR x=0,nx-1 DO img[x,y] = img[x,y]*c
        ENDFOR
    ENDIF
    img = FFT(img, /INVERSE, /OVERWRITE)
ENDIF
; cut out the central part of the rotated image
RETURN, REAL_PART(img[nx/4:nx/4 + s[0] -1, ny/4:ny/4 + s[1] - 1])
END
