FUNCTION ldrsALGOIMPL::Init, algorithm
IF NOT self->ldrsDIALOG::Init() THEN RETURN, 0
self.algorithm = algorithm
self.module = self.algorithm->GetModule()
self.recipe = self.module->GetRecipe()
self.dlgTitle = 'Algorithm Setup'
self.pauseFlag = 0
RETURN, 1
END

FUNCTION ldrsALGOIMPL::Locate, ft, FILENR=filenr, IMGNR=imgnr, DIRECT=direct, _REF_EXTRA=ex
IF NOT OBJ_VALID(ft) THEN RETURN, OBJ_NEW()
IF N_ELEMENTS(filenr) EQ 0 THEN filenr = 0
IF filenr GE ft->GetNFiles() THEN RETURN, OBJ_NEW()
file = ft->GetFile(filenr)
IF N_ELEMENTS(imgnr) EQ 0 THEN BEGIN
    cfg = file->GetFileCFG()
ENDIF ELSE BEGIN
    cfg = file->GetImageCFG(imgnr)
ENDELSE
IF NOT OBJ_VALID(cfg) THEN RETURN, OBJ_NEW()
IF N_ELEMENTS(direct) NE 0 THEN RETURN, cfg
RETURN, cfg->LocateByName(self.module->GetTag(), _EXTRA=ex)
END

FUNCTION ldrsALGOIMPL::Prepare
;PRINT, 'ldrsALGOIMPL::Prepare'
self.start_mem = MEMORY(/CURRENT)
self.pauseFlag = 0
IF self.optionsLoaded EQ 0 THEN self->Load
RETURN, 1
END

FUNCTION ldrsALGOIMPL::Execute
;PRINT, 'ldrsALGOIMPL::Execute'
RETURN, 1
END

FUNCTION ldrsALGOIMPL::Finish
;PRINT, 'ldrsALGOIMPL::Finish'
high_mem = MEMORY(/HIGHWATER)
end_mem = MEMORY(/CURRENT)
PRINT, 'memory start=', ROUND(self.start_mem/1024/1024), $
  ', high=', ROUND(high_mem/1024/1024), $
  ', end=', ROUND(end_mem/1024/1024), $
  ', needed=', ROUND((high_mem-self.start_mem)/1024/1024)
RETURN, 1
END

PRO ldrsALGOIMPL::ExecuteAlgorithm, ALL=all, SINGLE=single
IF NOT self->Prepare() THEN RETURN
IF NOT self->Execute() THEN RETURN
IF NOT self->Finish() THEN RETURN
END

PRO ldrsALGOIMPL::Pause
self.pauseFlag = 1
END

PRO ldrsALGOIMPL::Resume
IF NOT self.pauseFlag THEN RETURN
self.pauseFlag = 0
IF NOT self->Execute() THEN RETURN
IF NOT self->Finish() THEN RETURN
END

PRO ldrsALGOIMPL::Load, cfg
IF N_ELEMENTS(cfg) EQ 0 THEN cfg = self.module->GetCFG()
self->ldrsDIALOG::Load, cfg
END

PRO ldrsALGOIMPL::Store, cfg
IF N_ELEMENTS(cfg) EQ 0 THEN cfg = self.module->GetCFG()
self->ldrsDIALOG::Store, cfg
END

PRO ldrsALGOIMPL::CreateContent
self->ldrsDIALOG::CreateContent
END

PRO ldrsALGOIMPL::Edit, top
PRINT, 'ldrsALGOIMPL::Edit'
self->CreateDialog, top
IF self.optionsLoaded EQ 0 THEN self->Load
self->ReadStaticContent
self->ReadDynamicContent
self->ShowDialog
END

FUNCTION ldrsALGOIMPL::CheckInputFiles, filetype, minCard, maxCard
; check the cardinality
IF filetype->CheckCardinality(minCard, maxCard) EQ 0 THEN RETURN, 0
; check the existence of the files
RETURN, filetype->CheckExistance()
END

FUNCTION ldrsALGOIMPL::CheckOutputFiles, filetype, minCard, maxCard
; check the cardinality
RETURN, filetype->CheckCardinality(minCard, maxCard)
END

FUNCTION ldrsALGOIMPL::StoreOutput, outFT
nFiles = outFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    outFile = outFT->GetFile(iFile)
    IF NOT outFile->StoreData() EQ 0 THEN RETURN, 0
ENDFOR
RETURN, 1
END

PRO ldrsALGOIMPL__define
struct = { ldrsALGOIMPL, $
           INHERITS ldrsDIALOG, $
           recipe:OBJ_NEW(), $
           module:OBJ_NEW(), $
           algorithm:OBJ_NEW(), $
           start_mem:0L, $
           pauseFlag:0 $
         }
END
