FUNCTION ldrsDECONV_KHHBB::Init, algorithm
IF NOT self->ldrsBUILDINGBLOCK::Init(algorithm) THEN RETURN, 0
self.filltype = 0
self.regAlgo = 2
self.lambda = 1
self.betaMP = '1.0'
self.beta = 1.0
self.snr = 20000.0
self.blockSizeMP = '0.4'
self.blockSize = 0.4
self.maskSizeMP = '100.0'
self.maskSize = 100.0
RETURN, 1
END

FUNCTION ldrsDECONV_KHHBB::Prepare
PRINT, 'ldrsDECONV_KHHBB::Prepare'
IF NOT self->ldrsBUILDINGBLOCK::Prepare() THEN RETURN, 0
RETURN, 1
END

FUNCTION ldrsDECONV_KHHBB::Execute
COMMON ldrs, env
PRINT, 'ldrsDECONV_KHHBB::Execute: self.nIter=', self.nIter, ', self.pIter=', self.pIter
objFile = self.objFT->GetFile(0)
objName = objFile->GetName()
self.obj = objFile->GetData()
cfg = objFile->GetImageCFG(0)
objBgrd = cfg->GetFloat('BGRD', DEFAULT=0.0)
psfFile = self.psfFT->GetFile(0)
self.psf = psfFile->GetData()
psfName = psfFile->GetName()
recName = self.recFile->GetName()
nphot = TOTAL(*(self.obj))
PRINT, '  nphot= ', nphot
; building the mask
maskName=STRING(FILE_DIRNAME(objName, /MARK_DIRECTORY), 'mask.fits')
mask = FLTARR(self.nx, self.ny)
; create the ff mask from the psf
s = SIZE(*(self.psf), /DIMENSIONS)
hmask = ABS(FFT(*(self.psf)))
limit = MAX(hmask)*self.maskSize/100.0
hmask = SHIFT(hmask, s[0]/2, s[1]/2)
index = WHERE(hmask GT limit, count)
PRINT, '  count = ', count
IF count NE 0 THEN mask[index] = 1.0
mask = mask[self.nx/2:self.nx-1,*]
FITS_WRITE, maskName, mask, /NO_ABORT, MESSAGE=msg
IF msg NE '' THEN BEGIN
    PRINT, 'ldrsDECONV_KHHBB::Execute, ', maskName, ' errmsg=', msg
    RETURN, 0
ENDIF
cmdLine = STRING(FORMAT='("ldrsBB2.sh -img ", A, " -psf ", A, " -rec ", A, " -mask ", A, " -niter ", I, " -reg ", I, " 0 ", F, " 0 -threshold ", F, " ", I, " -block ", F, " -noise ", F, " 1 -det ", F, " -nphot ", E)', $
                 objName, psfName, recName, maskName, $
                 self.nIter, $
                 self.regAlgo, self.beta, $
                 self.threshold, self.filltype, $
                 self.blockSize, self.snr, sqrt(objBgrd), $
                 nphot)
PRINT, 'call external program with ', cmdLine
;SPAWN, cmdLine, result, errResult
;PRINT, result
;PRINT, errResult
;PRINT, 'read the reconstruction from file ', recName
res = self.recFile->LoadData(/FORCE)
self.rec = self.recFile->GetData()
env.recipe->Broadcast, 'FileLoaded', self.recFile
;LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.rec, NAME='Reconstruction'
;WAIT, 10
RETURN, 1
END

PRO ldrsDECONV_KHHBB::Load, cfg
self->ldrsBUILDINGBLOCK::Load, cfg
IF OBJ_VALID(cfg) THEN BEGIN
    self.filltype = cfg->GetInt('FILLTYPE', DEFAULT=0)
    self.regAlgo = cfg->GetInt('REG.ALGO', DEFAULT=0)
    self.lambda = cfg->GetInt('REG.LAMBDA', DEFAULT=1.0)
    self.betaMP = cfg->GetString('REG.BETA', DEFAULT='1.0')
    self.snr = cfg->GetFloat('SNR', DEFAULT=20000.0)
    self.blockSizeMP = cfg->GetString('BLOCKSIZE', DEFAULT='0.4')
    self.maskSizeMP = cfg->GetString('MASKSIZE', DEFAULT='100.0')
ENDIF ELSE BEGIN
    self.filltype = 0
    self.regAlgo = 0
    self.lambda = 1.0
    self.betaMP = '1.0'
    self.snr = 20000.0
    self.blockSizeMP = '0.4'
    self.maskSizeMP = '100.0'
ENDELSE
END

PRO ldrsDECONV_KHHBB::Store, cfg
self->ldrsBUILDINGBLOCK::Store, cfg
IF OBJ_VALID(cfg) THEN BEGIN
    cfg->SetInt, 'FILLTYPE', self.filltype
    cfg->SetInt, 'REG.ALGO', self.regAlgo
    cfg->SetInt, 'REG.LAMBDA', self.lambda
    cfg->SetString, 'REG.BETA', self.betaMP
    cfg->SetFloat, 'SNR', self.snr
    cfg->SetString, 'BLOCKSIZE', self.blockSizeMP
    cfg->SetString, 'MASKSIZE', self.maskSizeMP
ENDIF
END

; the number of parameters which can have multiple values which are
; used successively
FUNCTION ldrsDECONV_KHHBB::GetNoMP
RETURN, 3 + self->ldrsBUILDINGBLOCK::GetNoMP()
END

; short tag which is used as a prefix in the generated file name
FUNCTION ldrsDECONV_KHHBB::GetMPTag, nr
si = self->ldrsBUILDINGBLOCK::GetNoMP()
IF nr EQ si+0 THEN RETURN, 'b'
IF nr EQ si+1 THEN RETURN, 's'
IF nr EQ si+2 THEN RETURN, 'm'
RETURN, self->ldrsBUILDINGBLOCK::GetMPTag(nr)
END

; label whch is used in the user interface
FUNCTION ldrsDECONV_KHHBB::GetMPLabel, nr
si = self->ldrsBUILDINGBLOCK::GetNoMP()
IF nr EQ si+0 THEN RETURN, 'beta'
IF nr EQ si+1 THEN RETURN, 'blocksize'
IF nr EQ si+2 THEN RETURN, 'masksize'
RETURN, self->ldrsBUILDINGBLOCK::GetMPLabel(nr)
END

; get a string which contains all values of a specific parameter
FUNCTION ldrsDECONV_KHHBB::GetMPValues, nr
si = self->ldrsBUILDINGBLOCK::GetNoMP()
IF nr EQ si+0 THEN RETURN, self.betaMP
IF nr EQ si+1 THEN RETURN, self.blockSizeMP
IF nr EQ si+2 THEN RETURN, self.maskSizeMP
RETURN, self->ldrsBUILDINGBLOCK::GetMPValues(nr)
END

; set a parameter to a specific value which is givan as a string
PRO ldrsDECONV_KHHBB::SetMP, nr, val
si = self->ldrsBUILDINGBLOCK::GetNoMP()
IF nr LT si THEN self->ldrsBUILDINGBLOCK::SetMP, nr, val
IF nr EQ si+0 THEN self.beta = FLOAT(val)
IF nr EQ si+1 THEN self.blockSize = FLOAT(val)
IF nr EQ si+2 THEN self.maskSize = FLOAT(val)
END

PRO ldrsDECONV_KHHBB::ItemSelected, id, item
IF id EQ self.dFilltype THEN self.filltype = item
IF id EQ self.dRegAlgo THEN self.regAlgo = item
END

PRO ldrsDECONV_KHHBB::ReadDynamicContent
self->ldrsBUILDINGBLOCK::ReadDynamicContent
WIDGET_CONTROL, self.dFilltype, SET_COMBOBOX_SELECT=self.filltype
WIDGET_CONTROL, self.dRegAlgo, SET_COMBOBOX_SELECT=self.regAlgo
;WIDGET_CONTROL, self.dLambda, SET_VALUE=STRING(self.lambda)
WIDGET_CONTROL, self.dBeta, SET_VALUE=self.betaMP
WIDGET_CONTROL, self.dSNR, SET_VALUE=STRING(self.snr)
WIDGET_CONTROL, self.dBlockSize, SET_VALUE=self.blockSizeMP
WIDGET_CONTROL, self.dMaskSize, SET_VALUE=self.maskSizeMP
END

PRO ldrsDECONV_KHHBB::WriteDynamicContent
self->ldrsBUILDINGBLOCK::WriteDynamicContent
;WIDGET_CONTROL, self.dLambda, GET_VALUE=str
;self.lambda = FLOAT(str)
WIDGET_CONTROL, self.dBeta, GET_VALUE=str
self.betaMP = str
WIDGET_CONTROL, self.dSNR, GET_VALUE=str
self.snr = FLOAT(str)
WIDGET_CONTROL, self.dBlockSize, GET_VALUE=str
self.blockSizeMP = str
WIDGET_CONTROL, self.dMaskSize, GET_VALUE=str
self.maskSizeMP = str
END

PRO ldrsDECONV_KHHBB::CreateContent
self->ldrsBUILDINGBLOCK::CreateContent
row = self->GetFreeRow()
lab             = self->CreateLabel(0, row, 'Fill type', /HALIGN_RIGHT)
self.dFilltype  = self->CreateComboBox(2, row, ['All', 'Extrema'])
row++

lab             = self->CreateLabel(0, row, 'Regularization:', /HALIGN_RIGHT)
;lab             = self->CreateLabel(1, row, 'Algo', /HALIGN_RIGHT)
self.dRegAlgo   = self->CreateComboBox(2, row, ['None', 'Sum{|ok|^2}', 'Sum{ok*ln|ok|}', 'Sum{ln|ok|}'])
;lab           = self->CreateLabel(0, row, 'Lambda:')
;self.dLambda  = self->CreateEntry(1, row, VALUE='1.0')
;row++
lab             = self->CreateLabel(3, row, 'beta=', /HALIGN_RIGHT)
self.dBeta      = self->CreateEntry(4, row, VALUE='1.0')
row++
lab             = self->CreateLabel(0, row, 'SNR:', /HALIGN_RIGHT)
self.dSNR       = self->CreateEntry(2, row, VALUE='20000.0')
row++
lab             = self->CreateLabel(0, row, 'Blocksize:', /HALIGN_RIGHT)
self.dBlockSize     = self->CreateEntry(2, row, VALUE='20000.0')
row++
lab             = self->CreateLabel(0, row, 'Masksize[%]:', /HALIGN_RIGHT)
self.dMaskSize  = self->CreateEntry(2, row, VALUE='100.0')
row++
END

PRO ldrsDECONV_KHHBB__define
struct = { ldrsDECONV_KHHBB, $
           INHERITS ldrsBUILDINGBLOCK, $
           dFilltype:0, filltype:0, $
           dRegAlgo:0, regAlgo:0, $
           dLambda:0, lambda:1, $
           dBeta:0, betaMP:'', beta:1.0, $
           dSNR:0, snr:0.0, $
           dBlockSize:0, blockSizeMP:'', blockSize:0.4, $
           dMaskSize:0, maskSizeMP:'', maskSize:100.0, $
           psf:PTR_NEW(), $
           obj:PTR_NEW(), $
           mask:PTR_NEW() $
         }
END
