FUNCTION ldrsDECONV_SBB::Init, algorithm
IF NOT self->ldrsBUILDINGBLOCK::Init(algorithm) THEN RETURN, 0

RETURN, 1
END

FUNCTION ldrsDECONV_SBB::Prepare
;PRINT, 'ldrsDECONV_SBB::Prepare'
IF NOT self->ldrsBUILDINGBLOCK::Prepare() THEN RETURN, 0
; reduce the set of object images to one image
file = self.objFT->GetFile(0)
objImg = FLTARR(self.nx, self.ny)
data = file->GetData()
objImg += *data
self.objNPhot = TOTAL(objImg) ; store the number of photons in order to rescale the result
objImg /= self.objNPhot ; normalize the object image to an integral of 1.0
self.obj = PTR_NEW(objImg)
LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.obj, NAME='Object'
WAIT, 1
; and now reduce the set of psf images to one image
file = self.psfFT->GetFile(0)
psfImg = FLTARR(self.nx, self.ny)
data = file->GetData()
psfImg += *data
psfImg /= TOTAL(psfImg) ; normalize the psf to an integral of 1.0
self.psf = PTR_NEW(psfImg)
LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.psf, NAME='Psf'
WAIT, 1
; calculate the FFT for the psf
self.psfFft = PTR_NEW(FFT(*self.psf))
;self.psfFftAbs = PTR_NEW(ABS(*self.psfFft))
;LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.psfFftAbs, NAME='Psf FFT Absolute'
;WAIT, 1
;self.psfFftConj = PTR_NEW(CONJ(*self.psfFft))
; get the reconstructed image from the accompanying file
self.rec = self.recFile->GetData()
(*self.rec)[*] = 0.0
LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.rec, NAME='Reconstruction'
;WAIT, 10
RETURN, 1
END

FUNCTION ldrsDECONV_SBB::ExecuteStep
;PRINT, 'ldrsDECONV_SBB::ExecuteStep: self.iIter=', self.iIter
IF NOT self->ldrsBUILDINGBLOCK::ExecuteStep() THEN RETURN, 0
tmpArr = FFT(*self.rec)*(*self.psfFft)
tmpArr = ABS(FFT(tmpArr, /INVERSE))
tmpArr = SHIFT(tmpArr, self.nx/2, self.ny/2)
tot = TOTAL(tmpArr)
IF tot NE 0.0 THEN tmpArr /= -tot
tmpArr += *self.obj
peak = MAX(tmpArr)
indices = WHERE(tmpArr GE 0.01*self.threshold*peak)
;PRINT, '    peak=', peak, ', nblocks=', N_ELEMENTS(indices), ', first index=', indices[0]
;PRINT, '    indices=', indices
(*self.rec)[indices] += 1.0
RETURN, 1
END

PRO ldrsDECONV_SBB__define
struct = { ldrsDECONV_SBB, $
           INHERITS ldrsBUILDINGBLOCK, $
           psf:PTR_NEW(), $
           psfFft:PTR_NEW(), $
           psfFftConj:PTR_NEW(), $
           psfFftAbs:PTR_NEW(), $
           obj:PTR_NEW() $
         }
END
