FUNCTION ldrsDECONVOLVE::Init, algorithm
IF NOT self->ldrsALGOIMPL::Init(algorithm) THEN RETURN, 0
self.dlgTitle = 'Deconvolution Setup'
self.objFT = self.recipe->FindFiletype('TAR')
self.psfFT = self.recipe->FindFiletype('PSF')
self.rpsfFT = self.recipe->FindFiletype('RPSF')
self.rimgFT = self.recipe->FindFiletype('RIMG')
self.recFT = self.recipe->FindFiletype('REC')
self.imgFT = self.recipe->FindFiletype('IMG')
self.logFT = self.recipe->FindFiletype('LOG')
; set the default parameters
self.nIter = 1000
self.pIter = 10
; termination of the reconstruction
self.lastErrMeasure = 1.0
self.breakFlag = 0
RETURN, 1
END

FUNCTION ldrsDECONVOLVE::Prepare
;PRINT, 'ldrsDECONVOLVE::Prepare'
IF NOT self->ldrsALGOIMPL::Prepare() THEN RETURN, 0
IF NOT self->CheckInputFiles(self.objFT, 1, 10) THEN RETURN, 0
IF NOT self->CheckInputFiles(self.psfFT, 1, 10) THEN RETURN, 0
IF NOT self->CheckInputFiles(self.rpsfFT, 0, 1) THEN RETURN, 0
IF NOT self->CheckInputFiles(self.rimgFT, 0, 1) THEN RETURN, 0
IF self->GetNoMP() EQ 0 THEN BEGIN
    IF NOT self->CheckOutputFiles(self.recFT, 1, 1) THEN RETURN, 0
    IF NOT self->CheckOutputFiles(self.imgFT, 0, 1) THEN RETURN, 0
    IF NOT self->CheckOutputFiles(self.logFT, 0, 1) THEN RETURN, 0
ENDIF
self.nx = 0
self.ny = 0
self.nPA= 0
nFiles = self.objFT->GetNFiles()
lambda = 2.2
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.objFT->GetFile(iFile)
    cfg = file->GetFileCFG()
    hcfg = cfg->Locate('OBS.LAMBDA', /NO_CREATE)
    IF OBJ_VALID(hcfg) THEN lambda = hcfg->GetFloat('', DEFAULT=2.2)
    IF file->LoadData() EQ 0 THEN RETURN, 0
    self.nx = file->GetNX()
    self.ny = file->GetNY()
    self.nPA += file->GetNZ()
ENDFOR
nFiles = self.psfFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.psfFT->GetFile(iFile)
    IF file->LoadData() EQ 0 THEN RETURN, 0
ENDFOR
; get and load the optional reference psf
self.rpsfFile = OBJ_NEW()
nFiles = self.rpsfFT->GetNFiles()
IF nFiles EQ 1 THEN BEGIN
    self.rpsfFile = self.rpsfFT->GetFile(0)
    IF self.rpsfFile->LoadData() EQ 0 THEN RETURN, 0
ENDIF
; get and load the optional reference image
self.rimgFile = OBJ_NEW()
nFiles = self.rimgFT->GetNFiles()
IF nFiles EQ 1 THEN BEGIN
    self.rimgFile = self.rimgFT->GetFile(0)
    IF self.rimgFile->LoadData() EQ 0 THEN RETURN, 0
    self.lastOpt = PTR_NEW(FLTARR(self.nx, self.ny))
ENDIF
IF self->GetNoMP() EQ 0 THEN BEGIN
    ; here is only the code for algorithms without multiple-value parameters
    ; get and create the mandatory reconstruction file
    self.recFile = OBJ_NEW()
    nFiles = self.recFT->GetNFiles()
    IF nFiles EQ 1 THEN BEGIN
        self.recFile = self.recFT->GetFile(0)
        self.recFile->CreateData, self.nx, self.ny, 1, /FORCE
        cfg = self.recFile->GetFileCFG()
        cfg->SetFloat, 'OBS.LAMBDA', lambda
    ENDIF
    ; get and create the optional image file
    nFiles = self.imgFT->GetNFiles()
    IF nFiles EQ 1 THEN BEGIN
        self.imgFile = self.imgFT->GetFile(0)
        self.imgFile->CreateData, self.nx, self.ny, 1, /FORCE
        cfg = self.imgFile->GetFileCFG()
        cfg->SetFloat, 'OBS.LAMBDA', lambda
    ENDIF
    ; get and create the optional image file
    nFiles = self.logFT->GetNFiles()
    IF nFiles EQ 1 THEN BEGIN
        self.logFile = self.logFT->GetFile(0)
    ENDIF
ENDIF ELSE BEGIN
    ; here is only the code for algorithms with multiple-value parameters
    ; we expect that the method ExecuteSingleShot has set the
    ; recFile, imgFile, and logFile
    self.recFile->CreateData, self.nx, self.ny, 1, /FORCE
    cfg = self.recFile->GetFileCFG()
    cfg->SetFloat, 'OBS.LAMBDA', lambda
    IF OBJ_VALID(self.imgFile) THEN BEGIN
        self.imgFile->CreateData, self.nx, self.ny, 1, /FORCE
        cfg = self.imgFile->GetFileCFG()
        cfg->SetFloat, 'OBS.LAMBDA', lambda
    ENDIF
ENDELSE
; get some data
self->Load
self.iIter = 0
RETURN, 1
END

FUNCTION ldrsDECONVOLVE::ExecuteStep
RETURN, 1
END

PRO ldrsDECONVOLVE::FindBestReconstruction
IF NOT OBJ_VALID(self.rpsfFile) THEN RETURN
IF NOT OBJ_VALID(self.rimgFile) THEN RETURN
; convolve the reconstruction with the reference psf
result = self.rpsfFile->LoadData()
tmpRec = FFT(*self.rec)*FFT(*(self.rpsfFile->GetData()))
tmpRec = ABS(FFT(tmpRec, /DOUBLE, /INVERSE))
tmpRec = SHIFT(tmpRec, self.nx/2, self.ny/2)/TOTAL(tmpRec)
tmpRef = (*(self.rimgFile->GetData()))/TOTAL(*(self.rimgFile->GetData()))
; recenter reference image and reconstructed image by
; shifting the center of the reconstruction to the center of the reference
;s = SIZE(tmpRec, /DIMENSIONS)
;w = s[0]
;h = s[1]
;ipfft = FFT(tmpRec)
;hcx = ipfft[1,0]/ABS(ipfft[1,0])*COMPLEX(-1.0, 0.0)
;hcy = ipfft[0,1]/ABS(ipfft[0,1])*COMPLEX(-1.0, 0.0)
;xshiftRec = 0.5*ATAN(IMAGINARY(hcx), REAL_PART(hcx))/!PI*w;
;yshiftRec = 0.5*ATAN(IMAGINARY(hcy), REAL_PART(hcy))/!PI*h;
;ipfft = FFT(tmpRef)
;hcx = ipfft[1,0]/ABS(ipfft[1,0])*COMPLEX(-1.0, 0.0)
;hcy = ipfft[0,1]/ABS(ipfft[0,1])*COMPLEX(-1.0, 0.0)
;xshiftRef = 0.5*ATAN(IMAGINARY(hcx), REAL_PART(hcx))/!PI*w;
;yshiftRef = 0.5*ATAN(IMAGINARY(hcy), REAL_PART(hcy))/!PI*h;
;xshift = xshiftRef - xshiftRec
;yshift = yshiftRef - yshiftRec
;LDRS_TRANSLATE_FFT, tmpRec, tmpRec, TX=-xshift, TY=-yshift
; calculate the error
errVal = SQRT(TOTAL((tmpRec - tmpRef)^2)/TOTAL(tmpRef^2))
PRINT, '  ERROR: ', self.iIter, ' ', errVal ;, ' shifts=', xshift, yshift
IF errVal LT self.lastErrMeasure THEN BEGIN
    self.lastErrMeasure = errVal
    *self.lastOpt = *self.rec
ENDIF ELSE BEGIN
    IF (errVal - self.lastErrMeasure)/self.lastErrMeasure GT 0.02 THEN BEGIN
        ;self.breakFlag = 1
        PRINT, 'reconstruction (potentially) terminated after ', self.iIter, ' iterations'
    ENDIF
ENDELSE
END

FUNCTION ldrsDECONVOLVE::Execute
PRINT, 'ldrsDECONVOLVE::Execute: self.nIter=', self.nIter, ', self.pIter=', self.pIter
IF self.iIter EQ 0 THEN BEGIN
    IF NOT self->ldrsALGOIMPL::Execute() THEN RETURN, 0
ENDIF
WHILE (self.iIter LT self.nIter) AND NOT self.breakFlag DO BEGIN
    self.iIter++
    result = self->ExecuteStep()
    IF (self.iIter MOD self.pIter) EQ 0 THEN BEGIN
        PRINT, '  iIter=', self.iIter
        LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.rec, NAME='Reconstruction'
        self->FindBestReconstruction
    ENDIF
    IF self.pauseFlag THEN RETURN, 0
ENDWHILE
RETURN, 1
END

FUNCTION ldrsDECONVOLVE::Finish
;PRINT, 'ldrsDECONVOLVE::Finish'
IF NOT self->ldrsALGOIMPL::Finish() THEN RETURN, 0
; store the reconstruction
IF OBJ_VALID(self.rimgFile) THEN *self.rec = *self.lastOpt
IF self.recFile->StoreData() EQ 0 THEN RETURN, 0
; create the convolved image
IF OBJ_VALID(self.rpsfFile) AND OBJ_VALID(self.imgFile) THEN BEGIN
    result = self.rpsfFile->LoadData()
    tmpArr = FFT(*self.rec)*FFT(*(self.rpsfFile->GetData()))
    tmpArr = ABS(FFT(tmpArr, /DOUBLE, /INVERSE))
    tmpArr = SHIFT(tmpArr, self.nx/2, self.ny/2)
    tot = TOTAL(*(self.rpsfFile->GetData()))
    IF tot NE 0.0 THEN tmpArr /= tot
    ;tmpArr *= self.objNPhot
    *(self.imgFile->GetData()) = tmpArr
    result = self.imgFile->StoreData()
    LDRS_GUI_DISPATCH, 'ldrsIMAGE', 'ShowImage', IMAGE=self.imgFile->GetData(), NAME='Reconstructed Image'
ENDIF
self.iIter = 0
RETURN, 1
END

; the number of parameters which can have multiple values which are
; used successively
FUNCTION ldrsDECONVOLVE::GetNoMP
RETURN, 0
END

; short tag which is used as a prefix in the generated file name
FUNCTION ldrsDECONVOLVE::GetMPTag, nr
RETURN, ''
END

; label whch is used in the user interface
FUNCTION ldrsDECONVOLVE::GetMPLabel, nr
RETURN, ''
END

; get a string which contains all values of a specific parameter
FUNCTION ldrsDECONVOLVE::GetMPValues, nr
RETURN, ''
END

; set a parameter to a specific value which is givan as a string
PRO ldrsDECONVOLVE::SetMP, nr, val
END

PRO ldrsDECONVOLVE::CreateAllResultFiles, n, m, vals, counts
COMMON ldrs, env
recPre = self.recFT->GetPrefix()
imgPre = self.imgFT->GetPrefix()
logPre = self.logFT->GetPrefix()
recExt = self.recFT->GetExtension()
imgExt = self.imgFT->GetExtension()
logExt = self.logFT->GetExtension()
FOR i=0, n-1 DO BEGIN
    ; create the file name part containing the values
    fnpart = ''
    FOR j=m-1, 0, -1 DO BEGIN
        k = i MOD counts[j]
        fnpart = '_' + self->GetMPTag(j) + (*vals[j])[k] + fnpart
    ENDFOR
    IF recPre NE '' THEN BEGIN
        fname = recPre + fnpart + '.' + recExt
        file = self.recFT->GetFile(i)
        IF NOT OBJ_VALID(file) THEN file = self.recFT->CreateFile(fname)
        file->SetName, fname
    ENDIF
    IF imgPre NE '' THEN BEGIN
        fname = imgPre + fnpart + '.' + imgExt
        file = self.imgFT->GetFile(i)
        IF NOT OBJ_VALID(file) THEN file = self.imgFT->CreateFile(fname)
        file->SetName, fname
    ENDIF
    IF logPre NE '' THEN BEGIN
        fname = logPre + fnpart + '.' + logExt
        file = self.logFT->GetFile(i)
        IF NOT OBJ_VALID(file) THEN file = self.logFT->CreateFile(fname)
        file->SetName, fname
    ENDIF
ENDFOR
IF recPre NE '' THEN BEGIN
    env.recipe->Broadcast, 'FiletypeChanged', self.recFT
ENDIF
IF imgPre NE '' THEN BEGIN
    env.recipe->Broadcast, 'FiletypeChanged', self.imgFT
ENDIF
IF logPre NE '' THEN BEGIN
    env.recipe->Broadcast, 'FiletypeChanged', self.logFT
ENDIF
END

PRO ldrsDECONVOLVE::ExecuteSingleShot, n, vals, counts, i
IF i LT 0 THEN RETURN
; switch to the files for that sigle shot
self.recFile = self.recFT->GetFile(i)
self.imgFile = self.imgFT->GetFile(i)
self.logFile = self.logFT->GetFile(i)
; open the current log file
LDRS_LOG_CLEAR
IF OBJ_VALID(self.logFile) THEN BEGIN
    lfn = self.logFile->GetName()
    LDRS_LOG_APPEND, '### Starting logging into file ' + lfn
    LDRS_LOG_OPEN, lfn
ENDIF
LDRS_LOG_APPEND, '#############'
LDRS_LOG_APPEND, STRING(FORMAT='("### Performing a deconvolution, ", I0, " of ", I0, " alternatives, ", I0, " variable parameters:")', i+1, PRODUCT(counts), n)
; set the parameter to the requested values
FOR j=n-1, 0, -1 DO BEGIN
    k = i MOD counts[j]
    LDRS_LOG_APPEND, STRING('    ', self->GetMPLabel(j), ' = ', (*vals[j])[k])
    self->SetMP, j, (*vals[j])[k]
ENDFOR
self->ldrsALGOIMPL::ExecuteAlgorithm
LDRS_LOG_APPEND, '### finished'
; close the current log file
LDRS_LOG_CLOSE
END

PRO ldrsDECONVOLVE::ExecuteAlgorithm, ALL=all, SINGLE=single
; check if the algorithm can use multiple parameters
nMP = self->GetNoMP()
IF nMP EQ 0 THEN BEGIN
    ; no multiple parameters -> normal execution
    self->ldrsALGOIMPL::ExecuteAlgorithm
    RETURN
END
; use an array of pointers to arrays of strings which holds all
; parameter values as single strings
self->Load
mpVals = PTRARR(nMP)
mpCount = INTARR(nMP)
nRuns = 1
; get all values, splits them and store them
FOR i=0, nMP-1 DO BEGIN
    mpVals[i] = PTR_NEW(STRSPLIT(self->GetMPValues(i), ';', /EXTRACT))
    mpCount[i] = N_ELEMENTS(*mpVals[i])
    nRuns *= mpCount[i]
ENDFOR
self->CreateAllResultFiles, nRuns, nMP, mpVals, mpCount
IF N_ELEMENTS(all) NE 0 THEN BEGIN
    FOR i=0, nRuns-1 DO BEGIN
        IF self.pauseFlag THEN BREAK
        self->ExecuteSingleShot, nMP, mpVals, mpCount, i
    ENDFOR
    RETURN
ENDIF
IF N_ELEMENTS(single) NE 0 THEN BEGIN
    IF SIZE(single, /DIMENSIONS) EQ 0 THEN BEGIN
        self->ExecuteSingleShot, nMP, mpVals, mpCount, single
        RETURN
    ENDIF
    FOR i=0, N_ELEMENTS(list)-1 DO BEGIN
        IF self.pauseFlag THEN BREAK
        self->ExecuteSingleShot, nMP, mpVals, mpCount, list[i]
    ENDFOR
    RETURN
ENDIF
END

PRO ldrsDECONVOLVE::Load, cfg
self->ldrsALGOIMPL::Load, cfg
; new code: use the module configuration
hcfg = cfg
; old code: use the file configuration
;hcfg = self->Locate(self.recFT, /NO_CREATE)
IF OBJ_VALID(hcfg) THEN BEGIN
    self.nIter = hcfg->GetInt('NITER', DEFAULT=1000)
    self.pIter = hcfg->GetInt('PITER', DEFAULT=10)
ENDIF ELSE BEGIN
    self.nIter = 1000
    self.pIter = 10
ENDELSE
END

PRO ldrsDECONVOLVE::Store, cfg
self->ldrsALGOIMPL::Store, cfg
; new code: use the module configuration
hcfg = cfg
; old code: use the file configuration
;hcfg = self->Locate(self.recFT)
IF OBJ_VALID(hcfg) THEN BEGIN
    hcfg->SetInt, 'NITER', self.nIter
    hcfg->SetInt, 'PITER', self.pIter
ENDIF
END

PRO ldrsDECONVOLVE::ReadDynamicContent
WIDGET_CONTROL, self.ddNIter, SET_VALUE=STRING(self.nIter)
WIDGET_CONTROL, self.ddPIter, SET_VALUE=STRING(self.pIter)
END

PRO ldrsDECONVOLVE::WriteDynamicContent
WIDGET_CONTROL, self.ddNIter, GET_VALUE=str
self.nIter = FIX(str)
WIDGET_CONTROL, self.ddPIter, GET_VALUE=str
self.pIter = FIX(str)
END

PRO ldrsDECONVOLVE::CreateContent
self->ldrsALGOIMPL::CreateContent
row = self->GetFreeRow()
lab          = self->CreateLabel(0, row, 'Iterations:', /HALIGN_RIGHT)
self.ddNIter = self->CreateEntry(2, row, VALUE='10000')
row++

lab          = self->CreateLabel(0, row, 'Show every:', /HALIGN_RIGHT)
self.ddPIter = self->CreateEntry(2, row, VALUE='100')
row++
END

PRO ldrsDECONVOLVE__define
struct = { ldrsDECONVOLVE, $
           INHERITS ldrsALGOIMPL, $
           objFT:OBJ_NEW(), $
           psfFT:OBJ_NEW(), $
           rpsfFT:OBJ_NEW(), $
           rimgFT:OBJ_NEW(), $
           recFT:OBJ_NEW(), $
           imgFT:OBJ_NEW(), $
           logFT:OBJ_NEW(), $
           rpsfFile:OBJ_NEW(), $
           rimgFile:OBJ_NEW(), $
           recFile:OBJ_NEW(), $
           imgFile:OBJ_NEW(), $
           logFile:OBJ_NEW(), $
           ddNIter:0, ddPIter:0, $
           objNPhot:0.0, $
           nIter:0, pIter:0, iIter:0, $
           nx:0, ny:0, nPA:0, $
           rec:PTR_NEW(), $
           lastErrMeasure:0.0, $
           lastOpt:PTR_NEW(), $
           breakFlag:0 $
         }
END
