FUNCTION ldrsFRAME::Init, cfg
self.label = cfg->GetString('LABEL')
cfg->GetInt2D, 'SCRSIZE', w, h
self.scrSize = [w, h]
self.fontSize = cfg->GetFloat('FONTSIZE')
RETURN, 1
END

FUNCTION ldrsFRAME::GetGUI
RETURN, self.guiBase
END

PRO ldrsFRAME::ShowInfo, info
WIDGET_CONTROL, self.guiInfo, SET_VALUE=STRING(FORMAT='("Info: ", A)', info)
END

PRO ldrsFRAME::CreateMenu, base, cfg
cfgItem = cfg->GetFirst()
WHILE OBJ_VALID(cfgItem) DO BEGIN
    itemLabel = cfgItem->GetString('LABEL')
    itemSep = cfgItem->GetBoolean('SEP', DEFAULT=0)
    itemPro = cfgItem->GetString('PRO')
    IF itemPro EQ '' THEN BEGIN
        item = WIDGET_BUTTON(base, VALUE=itemLabel, EVENT_PRO='LDRS_NYI_EVENT', SEPARATOR=itemSep, UVALUE=self)
    END ELSE BEGIN
        item = WIDGET_BUTTON(base, VALUE=itemLabel, EVENT_PRO=itemPro, SEPARATOR=itemSep, UVALUE=self)
    END
    cfgItem = cfgItem->GetNext()
ENDWHILE
END

PRO ldrsFRAME::CreateMenuBar, menuBar, cfgFrame
; create the menu bar according to the specification in the
; configuration file
cfgMenu = cfgFrame->Locate('ITEM')
IF OBJ_VALID(cfgMenu) THEN BEGIN
    cfgMenu = cfgMenu->GetFirst()
    WHILE OBJ_VALID(cfgMenu) DO BEGIN
        menuLabel = cfgMenu->GetString('LABEL')
        menu = WIDGET_BUTTON(menuBar, VALUE=menuLabel, /MENU)
        cfgItem = cfgMenu->Locate('ITEM')
        self->CreateMenu, menu, cfgItem
        cfgMenu = cfgMenu->GetNext()
    ENDWHILE
END
END

PRO ldrsFRAME::CreatePanel, base, cfgPanel
COMMON ldrs, env
idlClass = cfgPanel->GetString('CLASS')
IF idlClass EQ '' THEN BEGIN
    horiz = cfgPanel->GetBoolean('HORIZONTAL')
    IF horiz THEN BEGIN
        panel = WIDGET_BASE(base, /ROW, SPACE=0, XPAD=0, YPAD=0)
    END ELSE BEGIN
        panel = WIDGET_BASE(base, /COLUMN, SPACE=0, XPAD=0, YPAD=0)
    END
    cfgChild = cfgPanel->Locate('PANEL')
    IF OBJ_VALID(cfgChild) THEN BEGIN
        cfgChild = cfgChild->GetFirst()
        WHILE OBJ_VALID(cfgChild) DO BEGIN
            self->CreatePanel, panel, cfgChild
            cfgChild = cfgChild->GetNext()
        ENDWHILE
    END
END ELSE BEGIN
    hpanel = OBJ_NEW(idlClass, cfgPanel, self)
    hpanel->CreateGUI, base, cfgPanel
    self.panels[self.nPanels] = hpanel
    self.nPanels += 1
    env.recipe->AddView, hpanel
END
END

PRO ldrsFRAME::CreateGUI, leader, cfgFrame
label = cfgFrame->GetString('LABEL')
IF leader EQ 0 THEN BEGIN
    self.guiBase = WIDGET_BASE(MBAR=menuBar, TITLE=label, /TLB_SIZE_EVENTS, UVALUE=self)
    leader = self.guiBase
END ELSE BEGIN
    self.guiBase = WIDGET_BASE(MBAR=menuBar, TITLE=label, /TLB_SIZE_EVENTS, UVALUE=self, GROUP_LEADER=leader)
END
self->CreateMenuBar, menubar, cfgFrame
self.guiMain = WIDGET_BASE(self.guiBase, /COLUMN, SPACE=0, XPAD=0, YPAD=0)
;horiz = cfgFrame->GetBoolean('HORIZONTAL')
;IF horiz THEN BEGIN
;    self.guiPanel = WIDGET_BASE(self.guiMain, /ROW, SPACE=0, XPAD=0, YPAD=0)
;END ELSE BEGIN
;    self.guiPanel = WIDGET_BASE(self.guiMain, /COLUMN, SPACE=0, XPAD=0, YPAD=0)
;END
;cfgPanel = cfgFrame->Locate('PANEL')
;IF OBJ_VALID(cfgPanel) THEN BEGIN
;    cfgPanel = cfgPanel->GetFirst()
;    WHILE OBJ_VALID(cfgPanel) DO BEGIN
;        self->CreatePanel, self.guiPanel, cfgPanel
;        cfgPanel = cfgPanel->GetNext()
;    ENDWHILE
self->CreatePanel, self.guiMain, cfgFrame
; info line
self.guiInfo = WIDGET_LABEL(self.guiMain, VALUE='Info:', /ALIGN_LEFT, /DYNAMIC_RESIZE)
END

PRO ldrsFRAME::RealizeGUI
;PRINT, 'ldrsFRAME::RealizeGUI'
WIDGET_CONTROL, self.guiBase, /REALIZE
FOR i = 0, self.nPanels - 1 DO BEGIN
    self.panels[i]->RealizeGUI
END
geo = WIDGET_INFO(self.guiBase, /GEOMETRY)
;PRINT, '   guiBase geo=', geo
self.minSize = [geo.scr_xsize, geo.scr_ysize]
self.scrSize = [geo.scr_xsize, geo.scr_ysize]
self.scrMBar = geo.scr_ysize - geo.ysize
geo = WIDGET_INFO(self.guiMain, /GEOMETRY)
;PRINT, '   guiMain geo=', geo
END

PRO ldrsFRAME::Resize, w, h
;PRINT, 'ldrsFRAME::Resize,', w, ',', h
;newW = w - self.scrDelta[0]
;newH = h - self.scrDelta[1]
newW = w
newH = h
;PRINT, '  stored old size:   ', self.scrSize[0], ',', self.scrSize[1]
;PRINT, '  requested new size:', newW, ',', newH
IF (newW LT self.minSize[0]) OR (newH LT self.minSize[1]) THEN BEGIN
    newW = MAX([self.minSize[0], newW])
    newH = MAX([self.minSize[1], newH])
    ;PRINT, '  modified new size: ', newW, ',', newH
    ;;WIDGET_CONTROL, self.guiBase, XSIZE=newW - self.scrDelta[0], YSIZE=newH - self.scrDelta[1]
    ;WIDGET_CONTROL, self.guiBase, SCR_XSIZE=newW, SCR_YSIZE=newH - self.scrMBar
    ;RETURN
ENDIF
resized=0
FOR i = 0, self.nPanels - 1 DO BEGIN
    IF self.panels[i]->Resize(newW - self.scrSize[0], newH - self.scrSize[1]) THEN resized = 1
END
IF NOT resized THEN BEGIN
    ;PRINT, '  no panel resized, keep the old size:', self.scrSize[0], ',', self.scrSize[1]
    geo = WIDGET_INFO(self.guiBase, /GEOMETRY)
    ;PRINT, '  geo before:', geo
    WIDGET_CONTROL, self.guiBase, $
      SCR_XSIZE=self.scrSize[0], SCR_YSIZE=self.scrSize[1] - self.scrMBar, $
      XSIZE=self.scrSize[0], YSIZE=self.scrSize[1] - self.scrMBar
    geo = WIDGET_INFO(self.guiBase, /GEOMETRY)
    ;PRINT, '  geo after: ', geo
ENDIF
; store the current size in instance variable 'scrSize'
geo = WIDGET_INFO(self.guiBase, /GEOMETRY)
self.scrSize = [geo.scr_xsize, geo.scr_ysize]
;PRINT, '  stored new size:   ', self.scrSize[0], ',', self.scrSize[1]
END

PRO ldrsFRAME::Dispatch, className, method, _REF_EXTRA=ex
FOR i = 0, self.nPanels - 1 DO BEGIN
    IF OBJ_ISA(self.panels[i], className) THEN CALL_METHOD, method, self.panels[i], _EXTRA=ex
ENDFOR
END

FUNCTION ldrsFRAME::Find, className
FOR i = 0, self.nPanels - 1 DO BEGIN
    IF OBJ_ISA(self.panels[i], className) THEN RETURN, self.panels[i]
ENDFOR
RETURN, OBJ_NEW()
END

PRO ldrsFRAME__define
  struct = { ldrsFRAME, $
             label:'', $
             minSize:[250, 250], $
             scrSize:[250, 250], $
             scrMBar:0, $
             fontSize:0.0, $
             guiBase:0, $
             guiMain:0, $
             guiPanel:0, $
             guiInfo:0, $
             nPanels:0, $
             panels:OBJARR(30) $
           }
END
