FUNCTION ldrsNORMALIZE::Init, algorithm
IF NOT self->ldrsALGOIMPL::Init(algorithm) THEN RETURN, 0
self.dlgTitle = 'Preprocessing Setup'
self.calFT = self.recipe->FindFiletype('CAL')
self.intFT = self.recipe->FindFiletype('PRE')
self.transformAlgo = 0
RETURN, 1
END

FUNCTION ldrsNORMALIZE::Prepare
PRINT, 'ldrsNORMALIZE::Prepare'
IF NOT self->ldrsALGOIMPL::Prepare() THEN RETURN, 0
IF NOT self->CheckInputFiles(self.calFT, 1, 10) THEN RETURN, 0
IF NOT self->CheckOutputFiles(self.intFT, 1, 10) THEN RETURN, 0
nFiles = self.calFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.calFT->GetFile(iFile)
    IF file->LoadData() EQ 0 THEN RETURN, 0
ENDFOR
nFiles = self.intFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.intFT->GetFile(iFile)
    ;file->CreateData, file->GetNX(), file->GetNY(), file->GetNZ()
ENDFOR
RETURN, 1
END

FUNCTION ldrsNORMALIZE::Execute
PRINT, 'ldrsNORMALIZE::Execute'
IF NOT self->ldrsALGOIMPL::Execute() THEN RETURN, 0
nFiles = self.intFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    calFile = self.calFT->GetFile(iFile)
    calData = calFile->GetData()
    calName = calFile->GetName()
    intFile = self.intFT->GetFile(iFile)
    intFile->CreateData, calFile->GetNX(), calFile->GetNY(), calFile->GetNZ()
    intData = intFile->GetData()
    intName = intFile->GetName()
    nImages = calFile->GetNImages()
    FOR iImage=0,nImages-1 DO BEGIN
        ; get the transformation parameters from FILE<i>.IMG<j>.PREP.*
        cfg = self->Locate(self.calFT, FILENR=iFile, IMGNR=iImage, /NO_CREATE)
        IF OBJ_VALID(cfg) THEN BEGIN
            rotAngle = cfg->GetFloat('ROTATE', DEFAULT=0.0)
            cfg->GetFloat2D, 'TRANSLATE', tX, tY, DEFAULT=[0.0, 0.0]
        ENDIF ELSE BEGIN
            rotAngle = 0.0
            translateX = 0.0
            translateY = 0.0
        ENDELSE
        PRINT, 'rotating image ', iImage, ' about ', rotAngle, ' degrees and translate it about ', tX, tY
        func = LDRS_GET_PROCEDURE('ITRANSFORM', self.transformAlgo)
        (*intData)[*,*,iImage] = CALL_FUNCTION(func, (*calData)[*,*,iImage], -rotAngle, tX, tY)
    ENDFOR
    LDRS_COPY_HEADER, calFile, intFile
ENDFOR
RETURN, 1
END

FUNCTION ldrsNORMALIZE::Finish
PRINT, 'ldrsNORMALIZE::Finish'
IF NOT self->ldrsALGOIMPL::Finish() THEN RETURN, 0
nFiles = self.intFT->GetNFiles()
FOR iFile=0,nFiles-1 DO BEGIN
    file = self.intFT->GetFile(iFile)
    IF file->StoreData() EQ 0 THEN RETURN, 0
ENDFOR
RETURN, 1
END

PRO ldrsNORMALIZE::ReadDynamicContent
PRINT, 'ldrsNORMALIZE::ReadDynamicContent: ', self.ndSelFileNr, ', ', self.ndImgNr, ', ', self.module->GetTag()
IF self.ndSelFileNr EQ -1 THEN RETURN
; get the position angle and reference flag from FILE<i>.IMG<j>.PA and FILE<i>.IMG<j>.ISREF
cfg = self->Locate(self.calFT, FILENR=self.ndSelFileNr, IMGNR=self.ndImgNr, /DIRECT, /NO_CREATE)
IF OBJ_VALID(cfg) THEN BEGIN
    isRef = cfg->GetBoolean('ISREF', DEFAULT=0)
ENDIF ELSE BEGIN
    isRef = 0
ENDELSE
; get scaling from     FILE<i>.IMG<j>.PREP.SCALE = "<sx> <sy>"
; get rotation from    FILE<i>.IMG<j>.PREP.ROTATE = "<r>"
; get translation from FILE<i>.IMG<j>.PREP.TRANSLATE = "<tx> <ty>"
cfg = self->Locate(self.calFT, FILENR=self.ndSelFileNr, IMGNR=self.ndImgNr, /NO_CREATE)
IF OBJ_VALID(cfg) THEN BEGIN
    cfg->GetFloat2D, 'SCALE', sx, sy, DEFAULT=[1.0, 1.0]
    r = cfg->GetFloat('ROTATE', DEFAULT=0.0)
    cfg->GetFloat2D, 'TRANSLATE', tx, ty, DEFAULT=[0.0, 0.0]
ENDIF ELSE BEGIN
    sx = 1.0
    sy = 1.0
    r = 0.0
    tx = 0.0
    ty = 0.0
ENDELSE
WIDGET_CONTROL, self.ndIsRef, SET_VALUE=[isRef]
WIDGET_CONTROL, self.ndTransformAlgo, SET_COMBOBOX_SELECT=self.transformAlgo
WIDGET_CONTROL, self.ndSX, SET_VALUE=STRING(sx)
WIDGET_CONTROL, self.ndSY, SET_VALUE=STRING(sy)
WIDGET_CONTROL, self.ndR, SET_VALUE=STRING(r)
WIDGET_CONTROL, self.ndTX, SET_VALUE=STRING(tx)
WIDGET_CONTROL, self.ndTY, SET_VALUE=STRING(ty)
END

PRO ldrsNORMALIZE::WriteDynamicContent
;PRINT, 'ldrsNORMALIZE::WriteParams: ', self.ndSelFileNr, ', ', self.ndImgNr, ', ', self.module->GetTag()
IF self.ndSelFileNr EQ -1 THEN RETURN
cfg = self->Locate(self.calFT, FILENR=self.ndSelFileNr, IMGNR=self.ndImgNr, /DIRECT, /NO_CREATE)
WIDGET_CONTROL, self.ndIsRef, GET_VALUE=flags
cfg->SetBoolean, 'ISREF', flags[0]
; store scaling as     FILE<i>.IMG<j>.PREP.SCALE = "<sx> <sy>"
; store rotation as    FILE<i>.IMG<j>.PREP.ROTATE = "<r>"
; store translation as FILE<i>.IMG<j>.PREP.TRANSLATE = "<tx> <ty>"
cfg = self->Locate(self.calFT, FILENR=self.ndSelFileNr, IMGNR=self.ndImgNr)
IF NOT OBJ_VALID(cfg) THEN RETURN
WIDGET_CONTROL, self.ndSX, GET_VALUE=str1
WIDGET_CONTROL, self.ndSY, GET_VALUE=str2
cfg->SetFloat2D, 'SCALE', FLOAT(str1), FLOAT(str2)
WIDGET_CONTROL, self.ndR, GET_VALUE=str
cfg->SetFloat, 'ROTATE', FLOAT(str)
WIDGET_CONTROL, self.ndTX, GET_VALUE=str1
WIDGET_CONTROL, self.ndTY, GET_VALUE=str2
cfg->SetFloat2D, 'TRANSLATE', FLOAT(str1), FLOAT(str2)
END

PRO ldrsNORMALIZE::ItemSelected, id, item
IF id EQ self.ndList THEN self->SelectFile, item
END

PRO ldrsNORMALIZE::SetTransformAlgo, id, item
self.transformAlgo = item
END

PRO ldrsNORMALIZE::SetEstimateAlgo, id, item
self.estimateAlgo = item
END

PRO ldrsNORMALIZE::ButtonPressed, id
CASE id OF
    self.ndPrevBtn: self->GotoPrev
    self.ndNextBtn: self->GotoNext
    self.ndEstimate: BEGIN
        ; get the current values from the dialog window
        WIDGET_CONTROL, self.ndR, GET_VALUE=str
        r = FLOAT(str)
        WIDGET_CONTROL, self.ndSX, GET_VALUE=str
        sx = FLOAT(str)
        WIDGET_CONTROL, self.ndSY, GET_VALUE=str
        sy = FLOAT(str)
        WIDGET_CONTROL, self.ndTX, GET_VALUE=str
        tx = FLOAT(str)
        WIDGET_CONTROL, self.ndTY, GET_VALUE=str
        ty = FLOAT(str)
        func = LDRS_GET_PROCEDURE('TRANSEST', self.estimateAlgo)
        cfg = self->Locate(self.calFT, FILENR=self.ndSelFileNr, IMGNR=self.ndImgNr, /DIRECT, /NO_CREATE)
        CALL_PROCEDURE, func, self.calFT, self.ndSelFileNr, self.ndImgNr, self.module->GetTag(), r, sx, sy, tx, ty
        WIDGET_CONTROL, self.ndSX, SET_VALUE=STRING(sx)
        WIDGET_CONTROL, self.ndSY, SET_VALUE=STRING(sy)
        WIDGET_CONTROL, self.ndR, SET_VALUE=STRING(r)
        WIDGET_CONTROL, self.ndTX, SET_VALUE=STRING(tx)
        WIDGET_CONTROL, self.ndTY, SET_VALUE=STRING(ty)
    END
    ELSE:
ENDCASE
END

PRO ldrsNORMALIZE::GotoNext
IF self.ndSelFileNr EQ -1 THEN RETURN
IF self.ndImgNr GE self.ndNImages - 1 THEN RETURN
self->WriteDynamicContent
self.ndImgNr += 1
WIDGET_CONTROL, self.ndNrTxt, SET_VALUE=STRING(FORMAT='(I0)', self.ndImgNr)
self->ReadDynamicContent
END

PRO ldrsNORMALIZE::GotoPrev
IF self.ndSelFileNr EQ -1 THEN RETURN
IF self.ndImgNr EQ 0 THEN RETURN
self->WriteDynamicContent
self.ndImgNr -= 1
WIDGET_CONTROL, self.ndNrTxt, SET_VALUE=STRING(FORMAT='(I0)', self.ndImgNr)
self->ReadDynamicContent
END

PRO ldrsNORMALIZE::SelectFile, nr
self->WriteDynamicContent
self.ndSelFileNr = nr
self.ndImgNr = 0
file = self.calFT->GetFile(self.ndSelFileNr)
res = file->Resize()
self.ndNImages = file->GetNImages()
WIDGET_CONTROL, self.ndNrTxt, SET_VALUE=STRING(FORMAT='(I0)', self.ndImgNr)
self->ReadDynamicContent
END

PRO ldrsNORMALIZE::CreateContent
self->ldrsALGOIMPL::CreateContent
row = self->GetFreeRow()

self.ndSelFileNr = -1
self.ndList    = self->CreateList(0, row, NX=5, YSIZE=4)
row++

lab            = self->CreateLabel(0, row,'Image:', /HALIGN_RIGHT)
self.ndPrevBtn = self->CreateButton(1, row, '<', /HALIGN_RIGHT)
self.ndNrTxt   = self->CreateEntry(2, row, VALUE='0')
self.ndNextBtn = self->CreateButton(3, row, '>', /HALIGN_LEFT)
self.ndIsRef   = self->CreateCheckbox(4, row, 'Reference', /HALIGN_LEFT)
row++

lab                  = self->CreateLabel(0, row, 'Transformation:', /HALIGN_RIGHT)
self.ndTransformAlgo = self->CreateProcedureMenu(2, row, 'ITRANSFORM', 'SetTransformAlgo')
row++

lab                 = self->CreateLabel(0, row, 'Parameters:', /HALIGN_RIGHT)
self.ndEstimateAlgo = self->CreateProcedureMenu(2, row, 'TRANSEST', 'SetEstimateAlgo', NX=2)
self.ndEstimate     = self->CreateButton(4, row, 'Estimate', NX=2, /HALIGN_LEFT)
row++

lab       = self->CreateLabel(0, row, 'Scale:', /HALIGN_RIGHT)
lab       = self->CreateLabel(1, row, 'sx=', /HALIGN_RIGHT)
self.ndSX = self->CreateEntry(2, row, VALUE=STRING(0.0))
lab       = self->CreateLabel(3, row, 'sy=', /HALIGN_RIGHT)
self.ndSY = self->CreateEntry(4, row, VALUE=STRING(0.0))
row++

lab       = self->CreateLabel(0, row, 'Rotate:', /HALIGN_RIGHT)
self.ndR  = self->CreateEntry(2, row, VALUE=STRING(0.0))
row++

lab       = self->CreateLabel(0, row, 'Translate:', /HALIGN_RIGHT)
lab       = self->CreateLabel(1, row, 'x=', /HALIGN_RIGHT)
self.ndTX = self->CreateEntry(2, row, VALUE=STRING(0.0))
lab       = self->CreateLabel(3, row, 'y=', /HALIGN_RIGHT)
self.ndTY = self->CreateEntry(4, row, VALUE=STRING(0.0))
row++
END

PRO ldrsNORMALIZE::ReadStaticContent
self.ndSelFileNr = -1
nFiles = self.calFT->GetNFiles()
IF nFiles EQ 0 THEN BEGIN
    senFlag=0
    arr = '<EMPTY>'
END ELSE BEGIN
    senFlag=1
    arr = STRARR(nFiles)
    FOR i = 0, nFiles - 1 DO BEGIN
        file = self.calFT->GetFile(i)
        arr[i] = file->GetName()
    ENDFOR
END
WIDGET_CONTROL, self.ndList, SET_VALUE=arr, SENSITIVE=senFlag
WIDGET_CONTROL, self.ndIsRef, SET_VALUE=[0]
WIDGET_CONTROL, self.ndEstimateAlgo, SET_COMBOBOX_SELECT=self.estimateAlgo
WIDGET_CONTROL, self.ndSX, SET_VALUE=STRING(0.0), SENSITIVE=senFlag
WIDGET_CONTROL, self.ndSY, SET_VALUE=STRING(0.0), SENSITIVE=senFlag
WIDGET_CONTROL, self.ndR, SET_VALUE=STRING(0.0), SENSITIVE=senFlag
WIDGET_CONTROL, self.ndTX, SET_VALUE=STRING(0.0), SENSITIVE=senFlag
WIDGET_CONTROL, self.ndTY, SET_VALUE=STRING(0.0), SENSITIVE=senFlag
END

PRO ldrsNORMALIZE::Load, cfg
self->ldrsALGOIMPL::Load, cfg
self.transformAlgo = cfg->GetInt('METHOD.TRANSFORM')
self.estimateAlgo = cfg->GetInt('METHOD.ESTIMATE')
END

PRO ldrsNORMALIZE::Store, cfg
self->ldrsALGOIMPL::Store, cfg
cfg->SetInt, 'METHOD.TRANSFORM', self.transformAlgo
cfg->SetInt, 'METHOD.ESTIMATE', self.estimateAlgo
END

PRO ldrsNORMALIZE__define
struct = { ldrsNORMALIZE, $
           INHERITS ldrsALGOIMPL, $
           calFT:OBJ_NEW(), $
           intFT:OBJ_NEW(), $
           ndTransformAlgo:0, transformAlgo:0, $
           ndList:0, $
           ndPrevBtn:0, $
           ndNrTxt:0, $
           ndNextBtn:0, $
           ndSelFileNr:0, $
           ndNImages:0, $
           ndImgNr:0, $
           ndIsRef:0, $
           ndEstimateAlgo:0, ndEstimate:0, estimateAlgo:0, $
           ndSX:0, ndSY:0, $
           ndR:0, $
           ndTX:0, ndTY:0 $
         }
END
